************************************************************************
* Advanced Dynamic Panel Data Methods
* Dr Sebastian Kripfganz
* University of Exeter Business School, Department of Economics
*
* Universidad de Salamanca
* July 17-19, 2023
*
* Methods to Analyze a Simple Dynamic Panel Data Model
************************************************************************
/*
This do-file provides an empirical example using the data set from the Arellano and Bond (1991) paper.

The aim is to estimate a dynamic employment equation:
  n_it = lambda n_i,t-1 + beta_1 w_it + beta_2 k_it + alpha_i + delta_t + epsilon_it
where
  n_it: log UK employment in company i at the end of year t
  w_it: log of real product wage (assumed to be strictly exogenous for this exercise)
  k_it: log of gross capital (assumed to be strictly exogenous for this exercise)
and
  alpha_i: company-specific intercept
  delta_t: time-specific effects (to be estimated with year dummy variables)
  epsilon_it: idiosyncratic error component

See Arellano and Bond (1991) for details on the data set.
See the presentation slides for details on the econometric methods.
See the Stata help files for details on command syntax and functionality.
*/

*----------------------------------------------------------------------*
* required additional Stata packages
net install xtdpdbc, from("http://www.kripfganz.de/stata/")			// package to compute the BC-MM estimator
net install xtdpdqml, from("http://www.kripfganz.de/stata/")		// package to compute the QML estimator

*----------------------------------------------------------------------*
* data set
clear all
webuse abdata								// Arellano and Bond (1991) data set

*----------------------------------------------------------------------*
* estimation
xtreg n L.n w k i.year, fe					// (biased) dynamic "fixed-effects" estimator

xtdpdbc n w k, teffects fe					// bias-corrected FE-MM estimator
estimates store bcfe						// store BC-MM estimation result for later use
xtdpdbc n w k, teffects re					// bias-corrected RE-MM estimator
estat overid								// overidentification test
estat hausman bcfe, df(2)					// generalized Hausman test (degrees of freedom need to be adjusted for the presence of time effects)

xtdpdqml n w k yr1978-yr1984, fe			// QML estimator

ivregress 2sls D.n (LD.n = L2D.n) D.(w k yr1979-yr1984), noconstant vce(robust)		// Anderson-Hsiao IV estimator (with instrument L2D.n and robust standard errors)
estat firststage																	// first-stage regression summary statistics
ivregress 2sls D.n (LD.n = L2.n) D.(w k yr1978-yr1984), noconstant vce(robust)		// Anderson-Hsiao IV estimator (with instrument L2.n and robust standard errors)
estat firststage																	// first-stage regression summary statistics
ivregress 2sls D.n (LD.n = L2D.n L2.n) D.(w k yr1979-yr1984), noconstant vce(robust)		// Anderson-Hsiao IV estimator (with instruments L2D.n L2.n and robust standard errors)
estat overid																				// overidentification test

*----------------------------------------------------------------------*
* serial correlation test
estimates restore bcfe						// restore BC-MM estimation results
estat serial, ar(1/3)						// Arellano-Bond serial correlation test (for serial correlation of order 1 to 3 in first-differenced errors)
