************************************************************************
* Advanced Dynamic Panel Data Methods
* Dr Sebastian Kripfganz
* University of Exeter Business School, Department of Economics
*
* Universidad de Salamanca
* July 17-19, 2023
*
* GMM Techniques for Dynamic Simultaneous Panel Data Models
************************************************************************
/*
This do-file provides an empirical example using the data set from the Arellano and Bond (1991) paper.

The aim is to estimate a dynamic employment equation:
  n_it = lambda n_i,t-1 + beta_1 w_it + beta_2 k_it + alpha_i + delta_t + epsilon_it
(possibly with distributed lags) where
  n_it: log UK employment in company i at the end of year t
  w_it: log of real product wage
  k_it: log of gross capital
and
  alpha_i: company-specific intercept
  delta_t: time-specific effects (to be estimated with year dummy variables)
  epsilon_it: idiosyncratic error component

See Arellano and Bond (1991) for details on the data set.
See the presentation slides for details on the econometric methods.
See the Stata help files for details on command syntax and functionality.
*/

*----------------------------------------------------------------------*
* required additional Stata packages
net install xtdpdgmm, from("http://www.kripfganz.de/stata/")		// package to compute the GMM estimators
ssc install underid													// package to compute underidentification tests

*----------------------------------------------------------------------*
* data set
clear all
webuse abdata								// Arellano and Bond (1991) data set

*----------------------------------------------------------------------*
* Arellano-Bond estimation (first-difference GMM)
xtdpdgmm n L.n w k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k, lagrange(0 .)) nolevel noconstant		// Arellano-Bond one-step GMM estimator with strictly exogenous regressors
xtdpdgmm n L.n w k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k, lagrange(1 .)) nolevel noconstant		// Arellano-Bond one-step GMM estimator with predetermined regressors
xtdpdgmm n L.n w k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k, lagrange(2 .)) nolevel noconstant		// Arellano-Bond one-step GMM estimator with endogenous regressors

xtdpdgmm n L.n w k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k, lagrange(2 .)) collapse nolevel noconstant		// Arellano-Bond one-step GMM estimator with endogenous regressors (collapsed instruments)
estimates store ab_col3

xtdpdgmm n L.n w k, teffects model(difference) gmmiv(n, lagrange(2 4)) gmmiv(w k, lagrange(2 4)) nolevel noconstant					// Arellano-Bond one-step GMM estimator with endogenous regressors (curtailed instruments)
xtdpdgmm n L.n w k, teffects model(difference) gmmiv(n, lagrange(2 4)) gmmiv(w k, lagrange(2 4)) collapse nolevel noconstant		// Arellano-Bond one-step GMM estimator with endogenous regressors (curtailed/collapsed instruments)

*----------------------------------------------------------------------*
* serial correlation test
estimates restore ab_col3
estimates replay ab_col3

estat serial, ar(1/3)						// Arellano-Bond serial correlation test (for serial correlation of order 1 to 3 in first-differenced errors)
estat serialpm, difference collapse			// Yamagata serial correlation test (for serial correlation of any order in first-differenced errors)
estat serialpm								// Jochmans serial correlation test (for serial correlation of any order)

*----------------------------------------------------------------------*
* Arellano-Bond estimation with distributed lags
xtdpdgmm n L.n L(0/1).w L(0/1).k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k, lagrange(2 .)) collapse nolevel noconstant		// Arellano-Bond one-step GMM estimator with endogenous regressors (collapsed instruments)
estimates store ab_col3

estat serial, ar(1/3)						// Arellano-Bond serial correlation test (for serial correlation of order 1 to 3 in first-differenced errors)
estat serialpm, difference collapse			// Yamagata serial correlation test (for serial correlation of any order in first-differenced errors)
estat serialpm								// Jochmans serial correlation test (for serial correlation of any order)

*----------------------------------------------------------------------*
* overidentification tests
estat overid								// Sargan overidentification test

xtdpdgmm n L.n L(0/1).w L(0/1).k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k, lagrange(2 .)) gmmiv(w k, lagrange(1 1)) collapse nolevel noconstant overid		// Arellano-Bond one-step GMM estimator with predetermined regressors (collapsed instruments)
estimates store ab_col2
estat overid ab_col3						// incremental overidentification test
estat overid, difference					// incremental overidentification test (guaranteed to be nonnegative)
estat hausman ab_col3						// generalized Hausman test

*----------------------------------------------------------------------*
* underidentification tests
estimates restore ab_col3
underid, nocollin jgmm2s					// underidentification test

*----------------------------------------------------------------------*
* Arellano-Bover estimation (forward-orthogonal deviations GMM)
xtdpdgmm n L.n L(0/1).w L(0/1).k, teffects model(fodev) gmmiv(n, lagrange(1 .)) gmmiv(w k, lagrange(1 .)) collapse nolevel noconstant		// Arellano-Bover one-step GMM estimator with endogenous regressors (collapsed instruments)

*----------------------------------------------------------------------*
* Ahn-Schmidt estimation (first-difference GMM with nonlinear moment functions)
xtdpdgmm n L.n L(0/1).w L(0/1).k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k, lagrange(2 .)) collapse nl(noserial) twostep vce(robust)		// Ahn-Schmidt two-step GMM estimator with endogenous regressors (collapsed instruments) and Windmeijer-corrected standard errors
estimates store as_col3

xtdpdgmm n L.n L(0/1).w L(0/1).k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k, lagrange(2 .)) collapse nl(iid) twostep vce(robust)		// Ahn-Schmidt two-step GMM estimator (assuming homoskedasticity) with endogenous regressors (collapsed instruments) and Windmeijer-corrected standard errors
estat overid as_col3						// incremental overidentification test
estat hausman as_col3						// generalized Hausman test

xtdpdgmm n L.n L(0/1).w L(0/1).k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k, lagrange(2 .)) collapse nl(noserial) igmm vce(robust)		// Ahn-Schmidt iterated GMM estimator with endogenous regressors (collapsed instruments) and Windmeijer-corrected standard errors
xtdpdgmm n L.n L(0/1).w L(0/1).k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k, lagrange(2 .)) collapse nl(noserial) cugmm vce(robust)		// Ahn-Schmidt continuously-updating GMM estimator with endogenous regressors (collapsed instruments) and Windmeijer-corrected standard errors

*----------------------------------------------------------------------*
* Blundell-Bond estimation (system GMM)
xtdpdgmm n L.n L(0/1).w L(0/1).k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k, lagrange(2 .)) gmmiv(n w k, difference lag(1 1) model(level)) collapse twostep vce(robust)		// Blundell-Bond two-step GMM estimator with endogenous regressors (collapsed instruments)
estat overid								// Hansen overidentification test
estat overid as_col3						// incremental overidentification test
estat hausman as_col3						// generalized Hausman test
underid, nocollin jgmm2s					// underidentification test

xtdpdgmm n L.n L(0/1).w L(0/1).k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k, lagrange(2 .)) gmmiv(n w k, difference lag(1 1) model(level)) collapse igmm vce(robust)		// Blundell-Bond iterated GMM estimator with endogenous regressors (collapsed instruments)
xtdpdgmm n L.n L(0/1).w L(0/1).k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k, lagrange(2 .)) gmmiv(n w k, difference lag(1 1) model(level)) collapse cugmm vce(robust)		// Blundell-Bond continuously-updating GMM estimator with endogenous regressors (collapsed instruments)

*----------------------------------------------------------------------*
* Blundell-Bond estimation (system GMM) with time-invariant industry dummy variables
xtdpdgmm n L.n L(0/1).w L(0/1).k i.ind, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k, lagrange(2 .)) gmmiv(n w k, difference lag(1 1) model(level)) iv(i.ind, model(level)) collapse twostep vce(robust)		// Blundell-Bond two-step GMM estimator with endogenous regressors (collapsed instruments)
