************************************************************************
* Advanced Dynamic Panel Data Methods
* Dr Sebastian Kripfganz
* University of Exeter Business School, Department of Economics
*
* Universidad de Salamanca
* July 17-19, 2023
*
* A Tentative Methodological Strategy for Panel Data Analysis
************************************************************************
/*
This do-file provides an empirical example using the data set from the Arellano and Bond (1991) paper.

The aim is to estimate a dynamic employment equation:
  n_it = lambda n_i,t-1 + beta_1 w_it + beta_2 k_it + alpha_i + delta_t + epsilon_it
(possibly with distributed lags) where
  n_it: log UK employment in company i at the end of year t
  w_it: log of real product wage
  k_it: log of gross capital
and
  alpha_i: company-specific intercept
  delta_t: time-specific effects (to be estimated with year dummy variables)
  epsilon_it: idiosyncratic error component

See Arellano and Bond (1991) for details on the data set.
See the presentation slides for details on the econometric methods.
See the Stata help files for details on command syntax and functionality.
*/

*----------------------------------------------------------------------*
* required additional Stata packages
net install xtdpdgmm, from("http://www.kripfganz.de/stata/")		// package to compute the GMM estimators
ssc install underid													// package to compute underidentification tests

*----------------------------------------------------------------------*
* data set
clear all
webuse abdata								// Arellano and Bond (1991) data set

*----------------------------------------------------------------------*
* initial candidate maintained statistical model with endogenous regressors
xtdpdgmm n L(1/2).n L(0/2).w L(0/2).k c.w#c.k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k c.w#c.k, lagrange(2 .)) collapse twostep nolevel noconstant vce(robust)		// Arellano-Bond two-step GMM estimator
estat overid								// Hansen overidentification test
estat serial, ar(1/3)						// Arellano-Bond serial correlation test (for serial correlation of order 1 to 3 in first-differenced errors)
estat serialpm, difference collapse			// Yamagata serial correlation test (for serial correlation of any order in first-differenced errors)
estat serialpm								// Jochmans serial correlation portmanteau test (for serial correlation of any order)
underid, nocollin jgmm2s					// underidentification test

// add nl(noserial) to gain identification strength
xtdpdgmm n L(1/2).n L(0/2).w L(0/2).k c.w#c.k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k c.w#c.k, lagrange(2 .)) nl(noserial) collapse twostep vce(robust)		// Ahn-Schmidt two-step GMM estimator
estat overid								// Hansen overidentification test
estat serial, ar(1/3)						// Arellano-Bond serial correlation test (for serial correlation of order 1 to 3 in first-differenced errors)
estat serialpm, difference collapse			// Yamagata serial correlation test (for serial correlation of any order in first-differenced errors)
estat serialpm								// Jochmans serial correlation portmanteau test (for serial correlation of any order)
estimates store msm							// store estimation results for current maintained statistical model

// drop L2.w due to high p-value
xtdpdgmm n L(1/2).n L(0/1).w L(0/2).k c.w#c.k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k c.w#c.k, lagrange(2 .)) nl(noserial) collapse twostep vce(robust)		// Ahn-Schmidt two-step GMM estimator
estat overid								// Hansen overidentification test
estat serial, ar(1/3)						// Arellano-Bond serial correlation test (for serial correlation of order 1 to 3 in first-differenced errors)
estat serialpm, difference collapse			// Yamagata serial correlation test (for serial correlation of any order in first-differenced errors)
estat serialpm								// Jochmans serial correlation portmanteau test (for serial correlation of any order)
estat mmsc msm								// Andrews-Lu model and moment selection criteria
estimates store msm							// store estimation results for current maintained statistical model

// possibly check for robustness to alternative instrument reduction techniques (skipped here)

*----------------------------------------------------------------------*
* maintained statistical model with predetermined regressors
// check for predeterminedness of w
xtdpdgmm n L(1/2).n L(0/1).w L(0/2).k c.w#c.k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k c.w#c.k, lagrange(2 .)) gmmiv(w, lagrange(1 1)) nl(noserial) collapse twostep vce(robust)		// Ahn-Schmidt two-step GMM estimator
estat overid								// Hansen overidentification test
estat overid msm							// incremental overidentification test
estat hausman msm							// generalized Hausman test
estat mmsc msm								// Andrews-Lu model and moment selection criteria

// check for predeterminedness of k
xtdpdgmm n L(1/2).n L(0/1).w L(0/2).k c.w#c.k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k c.w#c.k, lagrange(2 .)) gmmiv(k, lagrange(1 1)) nl(noserial) collapse twostep vce(robust)		// Ahn-Schmidt two-step GMM estimator
estat overid								// Hansen overidentification test
estat overid msm							// incremental overidentification test
estat hausman msm							// generalized Hausman test
estat mmsc msm								// Andrews-Lu model and moment selection criteria
estimates store msm							// store estimation results for current maintained statistical model

*----------------------------------------------------------------------*
* maintained statistical model with strictly exogenous regressors
// check for strict exogeneity of k
xtdpdgmm n L(1/2).n L(0/1).w L(0/2).k c.w#c.k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k c.w#c.k, lagrange(2 .)) gmmiv(k, lagrange(0 1)) nl(noserial) collapse twostep vce(robust)		// Ahn-Schmidt two-step GMM estimator
estat overid								// Hansen overidentification test
estat overid msm							// incremental overidentification test
estat hausman msm							// generalized Hausman test
estat mmsc msm								// Andrews-Lu model and moment selection criteria
estimates store msm							// store estimation results for current maintained statistical model

*----------------------------------------------------------------------*
* maintained statistical model with stronger initial-observations assumption (effect stationarity)
// check for effect stationarity of w
xtdpdgmm n L(1/2).n L(0/1).w L(0/2).k c.w#c.k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k c.w#c.k, lagrange(2 .)) gmmiv(k, lagrange(0 1)) gmmiv(w, difference lagrange(1 1) model(level)) collapse twostep vce(robust) overid		// Blundell-Bond two-step GMM estimator with endogenous regressors
estat overid								// Hansen overidentification test
estat overid, difference					// incremental overidentification test

// check for effect stationarity of k
xtdpdgmm n L(1/2).n L(0/1).w L(0/2).k c.w#c.k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k c.w#c.k, lagrange(2 .)) gmmiv(k, lagrange(0 1)) gmmiv(k, difference lagrange(0 0) model(level)) collapse twostep vce(robust) overid		// Blundell-Bond two-step GMM estimator with endogenous regressors
estat overid								// Hansen overidentification test
estat overid, difference					// incremental overidentification test

// check for joint effect stationarity
xtdpdgmm n L(1/2).n L(0/1).w L(0/2).k c.w#c.k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k c.w#c.k, lagrange(2 .)) gmmiv(k, lagrange(0 1)) gmmiv(n w c.w#c.k, difference lagrange(1 1) model(level)) gmmiv(w c.w#c.k, difference lagrange(0 0) model(level)) collapse twostep vce(robust)		// Blundell-Bond two-step GMM estimator with endogenous regressors
estat overid								// Hansen overidentification test
estat overid msm							// incremental overidentification test
estat hausman msm							// generalized Hausman test
underid, nocollin jgmm2s					// underidentification test
estat mmsc msm								// Andrews-Lu model and moment selection criteria
estimates store msm							// store estimation results for current maintained statistical model

*----------------------------------------------------------------------*
* maintained statistical model
// check for relevance of time-invariant industry dummy variables
xtdpdgmm n L(1/2).n L(0/1).w L(0/2).k c.w#c.k i.ind, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k c.w#c.k, lagrange(2 .)) gmmiv(k, lagrange(0 1)) gmmiv(n w c.w#c.k, difference lagrange(1 1) model(level)) gmmiv(w c.w#c.k, difference lagrange(0 0) model(level)) iv(i.ind, model(level)) collapse twostep vce(robust)		// Blundell-Bond two-step GMM estimator with endogenous regressors
estat mmsc msm								// Andrews-Lu model and moment selection criteria
testparm i.ind

// drop L2.n due to high p-value
xtdpdgmm n L(1/1).n L(0/1).w L(0/2).k c.w#c.k, teffects model(difference) gmmiv(n, lagrange(2 .)) gmmiv(w k c.w#c.k, lagrange(2 .)) gmmiv(k, lagrange(0 1)) gmmiv(n w c.w#c.k, difference lagrange(1 1) model(level)) gmmiv(w c.w#c.k, difference lagrange(0 0) model(level)) collapse twostep vce(robust)		// Blundell-Bond two-step GMM estimator with endogenous regressors
estat overid								// Hansen overidentification test
estat serial, ar(1/3)						// Arellano-Bond serial correlation test (for serial correlation of order 1 to 3 in first-differenced errors)
estat serialpm, difference collapse			// Yamagata serial correlation portmanteau test (for serial correlation of any order in first-differenced errors)
estat serialpm								// Jochmans serial correlation test (for serial correlation of any order)
underid, nocollin jgmm2s					// underidentification test
estat mmsc msm								// Andrews-Lu model and moment selection criteria

*----------------------------------------------------------------------*
* long-run multipliers for maintained statistical model
sum w, mean
local w_M = r(mean)
sum k, mean
local k_M = r(mean)
nlcom (w:(_b[w] + _b[L.w] + _b[c.w#c.k] * `k_M') / (1 - _b[L.n])) (k:(_b[k] + _b[L.k] + _b[L2.k] + _b[c.w#c.k] * `w_M') / (1 - _b[L.n]))
