************************************************************************
* Advanced Dynamic Panel Data Methods
* Dr Sebastian Kripfganz
* University of Exeter Business School, Department of Economics
*
* Universidad de Salamanca
* July 17-19, 2023
*
* A Detailed Empirical Illustration
************************************************************************
/*
This do-file provides an empirical example using happiness data from the German Socio-Economic Panel.

See notes attached to the Stata data set for detailed data sources.
See the Stata help files for details on command syntax and functionality.
*/

*----------------------------------------------------------------------*
* required additional Stata packages
net install xtdpdbc, from("http://www.kripfganz.de/stata/")			// package to compute the BC-MM estimator
net install xtdpdqml, from("http://www.kripfganz.de/stata/")		// package to compute the QML estimator
net install xtdpdgmm, from("http://www.kripfganz.de/stata/")		// package to compute the GMM estimators
net install xtseqreg, from("http://www.kripfganz.de/stata/")		// package to compute the Kripfganz-Schwarz two-stage estimator
ssc install underid													// package to compute underidentification tests

*----------------------------------------------------------------------*
* data set
clear all
use happy.dta
notes										// notes attached to the data set

*----------------------------------------------------------------------*
* (biased) RE/FE estimation
xtreg L(0/1).happy marry yrsmarried loghhinc age woman year, re		// "random-effects" estimator
xtreg L(0/1).happy marry yrsmarried loghhinc year, fe				// "fixed-effects" estimator

*----------------------------------------------------------------------*
* QML estimation
xtdpdqml happy marry yrsmarried loghhinc age woman year, re		// RE-QML estimation
estimates store qmlre
xtdpdqml happy marry yrsmarried loghhinc year, fe				// FE-QML estimation
estimates store qmlfe
hausman qmlfe qmlre, df(3)					// Hausman test for random vs. fixed effects

*----------------------------------------------------------------------*
* bias-corrected MM estimation
xtdpdbc happy marry yrsmarried loghhinc year, fe								// FE-BC-MM estimation
estimates store bcfe
xtdpdbc happy marry yrsmarried loghhinc age woman year, re eigtol(1e-10)		// RE-BC-MM estimation
estimates store bcre
hausman bcfe bcre, df(3)							// Hausman test for random vs. fixed effects
estat hausman bcfe (marry yrsmarried loghhinc)		// generalized Hausman test for random vs. fixed effects
estat overid								// Sargan-Hansen overidentification test for random vs. fixed effects
estat serial								// Arellano-Bond serial correlation test

*----------------------------------------------------------------------*
* bias-corrected MM estimation with second-order autoregressive dynamics
xtdpdbc happy marry yrsmarried loghhinc year, fe lags(2)								// FE-BC-MM estimation
estimates store bcfe2
xtdpdbc happy marry yrsmarried loghhinc age woman year, re eigtol(1e-10) lags(2)		// RE-BC-MM estimation
estimates store bcre2
hausman bcfe2 bcre2, df(3)							// Hausman test for random vs. fixed effects
estat hausman bcfe2 (marry yrsmarried loghhinc)		// generalized Hausman test for random vs. fixed effects
estat overid								// Sargan-Hansen overidentification test for random vs. fixed effects
estat serial								// Arellano-Bond serial correlation test

*----------------------------------------------------------------------*
* illustration of equivalence of first-difference and forward-orthogonal deviations transformation with all available instruments in balanced panels
xtdpdgmm L(0/2).happy marry yrsmarried loghhinc year, model(difference) gmmiv(happy, lagrange(2 .)) gmmiv(marry yrsmarried loghhinc year, lagrange(. .)) nolevel noconstant nofooter		// first differences
xtdpdgmm L(0/2).happy marry yrsmarried loghhinc year, model(fodev) gmmiv(happy, lagrange(1 .)) gmmiv(marry yrsmarried loghhinc year, lagrange(. .)) nolevel noconstant nofooter		// forward-orthogonal deviations

*----------------------------------------------------------------------*
* Arellano-Bond GMM estimation (first-difference transformation)
xtdpdgmm L(0/2).happy marry yrsmarried loghhinc year, model(difference) gmmiv(happy, lagrange(2 .)) gmmiv(marry yrsmarried loghhinc, lagrange(2 .)) iv(year, difference) nolevel noconstant		// all variables endogenous (besides linear time trend)

xtdpdgmm L(0/2).happy marry yrsmarried loghhinc year, model(difference) gmmiv(happy, lagrange(2 .)) gmmiv(marry yrsmarried loghhinc, lagrange(2 .)) iv(year, difference) twostep nolevel noconstant vce(robust)		// two-step estimation
xtdpdgmm L(0/2).happy marry yrsmarried loghhinc year, model(difference) gmmiv(happy, lagrange(2 .)) gmmiv(marry yrsmarried loghhinc, lagrange(2 .)) iv(year, difference) igmm nolevel noconstant vce(robust)		// iterated estimation
xtdpdgmm L(0/2).happy marry yrsmarried loghhinc year, model(difference) gmmiv(happy, lagrange(2 .)) gmmiv(marry yrsmarried loghhinc, lagrange(2 .)) iv(year, difference) cugmm nolevel noconstant vce(robust)		// continuously-updating estimation

xtdpdgmm L(0/2).happy marry yrsmarried loghhinc year, model(difference) gmmiv(happy, lagrange(2 .)) gmmiv(marry yrsmarried loghhinc, lagrange(2 .)) iv(year, difference) collapse twostep nolevel noconstant vce(robust)		// collapsed instruments
xtdpdgmm L(0/2).happy marry yrsmarried loghhinc year, model(difference) gmmiv(happy, lagrange(2 5)) gmmiv(marry yrsmarried loghhinc, lagrange(2 5)) iv(year, difference) twostep nolevel noconstant vce(robust)		// curtailed instruments
xtdpdgmm L(0/2).happy marry yrsmarried loghhinc year, model(difference) gmmiv(happy, lagrange(2 5)) gmmiv(marry yrsmarried loghhinc, lagrange(2 5)) iv(year, difference) collapse twostep nolevel noconstant vce(robust)		// curtailed and collapsed instruments
estimates store msm
estat serial								// Arellano-Bond serial correlation test
estat serialpm								// Jochmans serial-correlation portmanteau test
estat overid								// Hansen overidentification test

xtdpdgmm L(0/2).happy marry yrsmarried loghhinc year, model(difference) gmmiv(happy, lagrange(2 5)) gmmiv(marry yrsmarried loghhinc, lagrange(2 5)) gmmiv(marry, lagrange(1 1)) iv(year, difference) collapse twostep nolevel noconstant vce(robust)		// marriage treated as predetermined
estat overid msm							// incremental overidentification test
estat overid								// Hansen overidentification test
underid, sw									// Sanderson-Windmeijer underidentification tests

*----------------------------------------------------------------------*
* Arellano-Bover GMM estimation (forward-orthogonal deviations)
xtdpdgmm L(0/2).happy marry yrsmarried loghhinc year, model(fodev) gmmiv(happy, lagrange(1 5)) gmmiv(marry yrsmarried loghhinc, lagrange(1 5)) iv(year) collapse twostep vce(robust)		// curtailed and collapsed instruments
underid, sw									// Sanderson-Windmeijer underidentification tests

*----------------------------------------------------------------------*
* Ahn-Schmidt GMM estimation with nonlinear moment functions
xtdpdgmm L(0/2).happy marry yrsmarried loghhinc year, model(difference) gmmiv(happy, lagrange(2 5)) gmmiv(marry yrsmarried loghhinc, lagrange(2 5)) iv(year, difference) nl(noserial, nocollapse) collapse twostep vce(robust)		// with nonlinear moment conditions valid under no serial correlation
estimates store msm

xtdpdgmm L(0/2).happy marry yrsmarried loghhinc year, model(difference) gmmiv(happy, lagrange(2 5)) gmmiv(marry yrsmarried loghhinc, lagrange(2 5)) iv(year, difference) nl(iid, nocollapse) collapse twostep vce(robust)		// with nonlinear moment conditions valid under homoskedasticity and no serial correlation
estat overid msm							// incremental overidentification test for homoskedasticity versus heteroskedasticity
estat hausman msm							// generalized Hausman test for homoskedasticity versus heteroskedasticity

*----------------------------------------------------------------------*
* Blundell-Bond "system GMM" estimation
xtdpdgmm L(0/2).happy marry yrsmarried loghhinc year, model(difference) gmmiv(happy, lagrange(2 5)) gmmiv(marry yrsmarried loghhinc, lagrange(2 5)) iv(year, difference) gmmiv(happy marry yrsmarried loghhinc, lagrange(1 1) difference model(level)) collapse twostep vce(robust)
underid, sw									// Sanderson-Windmeijer underidentification tests
estat overid msm							// incremental overidentification test

xtdpdgmm L(0/2).happy marry yrsmarried loghhinc year, model(difference) gmmiv(happy, lagrange(2 5)) gmmiv(marry yrsmarried loghhinc, lagrange(2 5)) iv(year, difference) gmmiv(marry yrsmarried, lagrange(1 1) difference model(level)) collapse twostep vce(robust)
estat overid msm							// incremental overidentification test

*----------------------------------------------------------------------*
* Arellano-Bond estimation (alternative model specification)
xtdpdgmm L(0/2).happy marry yrsmarried loghhinc year, model(difference) gmmiv(happy, lagrange(2 5)) gmmiv(loghhinc, lagrange(2 5)) gmmiv(marry yrsmarried, lagrange(0 5)) iv(year, difference) collapse twostep vce(robust)		// marriage treated as exogenous
underid, sw									// Sanderson-Windmeijer underidentification tests
estat overid								// Hansen overidentification test

*----------------------------------------------------------------------*
* GMM estimation with time-invariant regressor
xtdpdgmm L(0/2).happy marry yrsmarried loghhinc woman year, model(difference) gmmiv(happy, lagrange(2 5)) gmmiv(loghhinc, lagrange(2 5)) gmmiv(marry yrsmarried, lagrange(0 5)) iv(year, difference) iv(woman, model(level)) collapse twostep vce(robust)

xtdpdgmm L(0/2).happy marry yrsmarried loghhinc year, model(difference) gmmiv(happy, lagrange(2 5)) gmmiv(loghhinc, lagrange(2 5)) gmmiv(marry yrsmarried, lagrange(0 5)) iv(year, difference) collapse twostep vce(robust) aux		// first stage of two-stage Kripfganz-Schwarz estimation
xtseqreg happy (L(1/2).happy marry yrsmarried loghhinc year) woman, first(, copy) vce(robust)		// second stage of two-stage Kripfganz-Schwarz estimation
